@@ -1,5 +1,4 @@
-import os, random, sys, time, csv, pickle, re, pkg_resources
-import mido
+import os, random, sys, time, csv, pickle, re, pkg_resources, mido
 os.environ['PYGAME_HIDE_SUPPORT_PROMPT']="hide"
 from tkinter import StringVar, DoubleVar, Tk, Label, Entry, Button, OptionMenu, \
 Checkbutton, Message, Menu, IntVar, Scale, HORIZONTAL, simpledialog, messagebox, Toplevel
@@ -37,7 +36,7 @@ def init_entry(fn):
 	entry.set(fn)
 	return entry
 
-def stack_videos(videos,fn='output.mp4'):
+def stack_videos(videos,fn):
 	'''
 	Stacks .mp4 videos horizontally (and combines audio)
 	'''
@@ -103,7 +102,7 @@ class GUI(Tk):
 		Sends message through print if no GUI, through self.status if GUI is running
 		'''
 		if self.display:
-			self.status['text']='♫ '+msg
+			self.status['text']=msg
 		else:
 			print(msg)
 
@@ -148,7 +147,7 @@ class GUI(Tk):
 		for k in data_unk.keys():
 			if k in ('__header__', '__version__', '__globals__'):
 				continue
-			elif len(data_unk[k])==1:
+			elif len(data_unk[k].flatten())==1:
 				data['fr']=float(data_unk[k])
 			elif data_unk[k].ndim==2:
 				data['H']=data_unk[k]
@@ -240,6 +239,7 @@ class GUI(Tk):
 				tmpH=self.data['H_pp'].T
 
 			self.Hax1.cla()
+			self.Hax1.set_title('Temporal Data (H)')
 			self.Hax1.plot(tmpH,linewidth=.5)
 			for i,j in enumerate(self.Hax1.lines):
 				j.set_color(self.cmap[i])
@@ -441,7 +441,7 @@ class GUI(Tk):
 				H_rs[i,:]=interp1d(t1,self.data['H_pp'][i,:])(t2)
 			for chan in range(nchan):
 				track.append(mido.Message('program_change', channel=chan, program=sound_presets[self.cfg['sound_preset']], time=0))
-				track.append(mido.Message('note_on', channel=chan, note=self.keys[chan], time=0))
+				track.append(mido.Message('note_on', channel=chan, note=self.keys[chan], time=0, velocity=127))
 				track.append(mido.Message('control_change', channel=chan, control=7, value=0, time=0))
 			for i in range(ns):
 				for chan in range(nchan):
@@ -500,7 +500,7 @@ class GUI(Tk):
 		Merges video and audio with ffmpeg
 		'''
 		fn=os.path.join(self.cfg['save_path'],self.cfg['file_out'])
-		cmd='ffmpeg -hide_banner -loglevel warning -y -i {} -i {} -c:v copy -c:a aac {}'.format(fn+'.mp4',fn+'.wav',fn+'_AV.mp4')
+		cmd='ffmpeg -hide_banner -loglevel warning -y -i {} -i {} -c:a aac -map 0:v:0 -map 1:a:0 {}'.format(fn+'.mp4',fn+'.wav',fn+'_AV.mp4')
 		os.system(cmd)
 		self.message(f'A/V file written to {self.cfg["save_path"]}')
 		return self
@@ -683,11 +683,10 @@ class GUI(Tk):
 		'''
 		# H
 		if 'H_pp' in self.data:
-			self.figH=plt.Figure(figsize=(6,6), dpi=100, tight_layout=True)
+			self.figH=plt.Figure(figsize=(7,6), dpi=100, tight_layout=True)
 			self.Hax1=self.figH.add_subplot(211)
 			self.Hax2=self.figH.add_subplot(212)
-			self.Hax1.set_title('Temporal Data (H)')
-			self.Hax2.set_title('Audio Preview (H\')')
+			self.Hax2.set_title('Audio Preview')
 			self.canvas_H=FigureCanvasTkAgg(self.figH, master=self)
 			self.canvas_H.get_tk_widget().grid(row=1,column=7,rowspan=30,columnspan=10)
 			bg=self.status.winfo_rgb(self['bg'])
@@ -697,7 +696,7 @@ class GUI(Tk):
 			self.Hax1.spines['bottom'].set_visible(False)
 			self.Hax1.spines['right'].set_visible(False)
 			self.Hax1.yaxis.tick_right()
-			self.Hax1.yaxis.set_label_position("right")
+			self.Hax1.yaxis.set_label_position('right')
 			self.Hax1.tick_params(axis='x',which='both',bottom=False, top=False, labelbottom=False, right=False)
 			self.Hax2.set(xlabel='time (sec)',ylabel='Component #')
 			self.Hax2.spines['left'].set_visible(False)
@@ -705,7 +704,7 @@ class GUI(Tk):
 			self.Hax2.spines['bottom'].set_visible(False)
 			self.Hax2.spines['right'].set_visible(False)
 			self.Hax2.yaxis.tick_right()
-			self.Hax2.yaxis.set_label_position("right")
+			self.Hax2.yaxis.set_label_position('right')
 			# Checkbox
 			Checkbutton(self, text="Offset H",command=self.refresh_GUI,variable=self.offsetH).grid(row=1,rowspan=1,column=16)
 		
@@ -745,18 +744,18 @@ class GUI(Tk):
 		self.comps_to_show_entry=Entry(textvariable=self.comps_to_show,width=15,justify='center')
 		self.comps_to_show_entry.grid(row=30, column=6, columnspan=1,sticky='W')
 		
-	def process_raw(self,file_in=None,n_clusters=None,frame_rate=None,save=False):
+	def process_raw(self,data=None,n_clusters=None,frame_rate=None,save=False,file_in=None):
 		'''
 		Decomposes raw dataset. Can be used in two ways: as a part of the 
 		GUI class for immediate processing (e.g. process_raw().write_AV()),
 		or as a method to save a new dataset. 
 		'''
-		if filein is None:
-			filein=uiopen(title='Select .mat file for import',filetypes=[('.mat files','*.mat')])
-		if filein=='.':
-			return
-		dh, var=loadmat(file_in),whosmat(file_in)
-		data=dh[var[0][0]]
+		if data is None:
+			file_in=uiopen(title='Select .mat file for import',filetypes=[('.mat files','*.mat')])
+			if filein=='.':
+				return
+			dh, var=loadmat(file_in),whosmat(file_in)
+			data=dh[var[0][0]]
 		sh=data.shape
 		if len(sh) != 3:
 			self.message('ERROR: input dataset is not 3D.')
@@ -785,27 +784,29 @@ class GUI(Tk):
 		for i in range(len(nnidx)):
 			W[nnidx[i],:]=nnls(H.T,data_nn[i,:])[0]
 		# Sort bottom to top
-		xc,yc=[], []
-		(X,Y)=np.meshgrid(range(sh[0]),range(sh[1]))
+		xc,yc=[],[]
+		(X,Y)=np.meshgrid(range(sh[1]),range(sh[0]))
 		for i in range(len(W.T)):
 			Wtmp=W[:,i].reshape(sh[0],sh[1])
 			xc.append((X*Wtmp).sum() / Wtmp.sum().astype("float"))
 			yc.append((Y*Wtmp).sum() / Wtmp.sum().astype("float"))
-		I=np.argsort(yc).reverse() # Reverse orders from bottom to top
-		W, H=W[:,I],H[I,:]
+		I=np.argsort(yc[::-1]) # Reverse orders from bottom to top
+		W,H=W[:,I],H[I,:]
 		print('done.')
 		
 		# Assign variables and save
 		self.data={}
 		self.data['H']=H
 		self.data['W']=W.reshape(sh[0],sh[1],n_clusters)
-		self.data['W_shape']=self.data['W'].shape
+		#self.data['W_shape']=self.data['W'].shape.flatten()
 		if frame_rate==[]:
 			self.data['fr']=10
 			print('No fr given. Defaulting to 10')
 		else:
 			self.data['fr']=frame_rate
 		if save:
+			if file_in is None:
+				file_in='data.mat'
 			fn=file_in.replace('.mat','_decomp.mat')
 			savemat(fn,self.data)
 			self.message(f'Decomposed data file saved to {fn}')
